﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8.5
@EndOfDesignText@
Sub Class_Globals
    Private mCallBack   		            								As Object	= Null
    Private mCallEventPrefix                   								As String	= ""

	Private mConnected														As Boolean  = False
			
	Private	mSocket															As Socket 	
	Private mAStream 														As AsyncStreams

	'-----------------------------------------------------------------------------------------------------------------------------------------
	'		IP Address below is the Address on router that Onkyo is using
	'-----------------------------------------------------------------------------------------------------------------------------------------
	Private mIPAddress														As String	= "192.168.1.12"
	
	'-----------------------------------------------------------------------------------------------------------------------------------------
	'		Onkyo connect port is 60128 
	'-----------------------------------------------------------------------------------------------------------------------------------------
	Private mPort															As Int		= 60128						'8080		'  was 23	
	Private mConnectTries													As Int		= 0

	
'---------------------------------------------------------------------------------------------------------------------------------------------
'	Here Is what works on my Onkyo receiver For "POWER OFF" And follows the ISCP spec (see linked diagram also)
'	POWER_OFF: 
'---------------------------------------------------------------------------------------------------------------------------------------------
' 	I  S  C  P        Hdr Size        Data Size     Vers,Rsrvd       !  1     P  W  R  0  0       EOL
'	73;83;67;80;       0;0;0;16;       0;0;0;8;      1;0;0;0;        33;49;   80;87;82;48;48;     13
'	<-----------Header size = 16----------------------------->        <--------Data size = 8--------->
'
'---------------------------------------------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------------------------------------------
'	In previous years Byte 	needed to be 8 for the ISCP and Header Size
'		AFTER 2016 the temination chars changes and byte 12 the Size needed to be 10
'---------------------------------------------------------------------------------------------------------------------------------------------
' 	The secret sauce For the 2016+ models seems To be terminating characters 26,13,10  And respective eISCP data size in byte number 12.
'
'	Power On:   73;83;67;80;0;0;0;16;0;0;0;10;1;0;0;0;33;49;80;87;82;48;48;26;13;10
'	Power Off:   73;83;67;80;0;0;0;16;0;0;0;10;1;0;0;0;33;49;80;87;82;48;49;26;13;10 
'---------------------------------------------------------------------------------------------------------------------------------------------

	Private mISCP															As String   = "ISCP"
	
	Private mHeader 														As String 	=   mISCP								 	_
																							&Chr(0)  &Chr(0)  &Chr(0)  &Chr(16) 	_
																							&Chr(0)  &Chr(0)  &Chr(0)  &Chr(10) 	_
																							&Chr(1)  &Chr(0)  &Chr(0)  &Chr(0)
																						
	Private mDeviceID													 	As String 	=  	 Chr(33) &Chr(49)

	Private mTrailer														As String 	=	 Chr(0x1A) &Chr(13) &Chr(10)
	
'	Private mPendingResponses												As String
	
	
	Private mListeningMode													As String	= ""
	
	Private Const DEFINE_Status_Request										As String 	= "QSTN"			' Master Volume Status

	Private Const DEFINE_Power_StandBy										As String 	= "00"				' Power Standby
	Private Const DEFINE_Power_On											As String 	= "01"				' Power On
	
	Private Const DEFINE_Volume_Up											As String 	= "UP"				' Volume Up
	Private Const DEFINE_Volume_Down										As String 	= "DOWN"			' Volume Down

	Private Const DEFINE_Master_Power										As String 	= "PWR"				' Master Power 	
	Private Const DEFINE_Master_Volume										As String 	= "MVL"				' Master Volume 
	Private Const DEFINE_Master_InputSelector								As String 	= "SLI"				' Master Input Selector
	
	Private Const DEFINE_Zone2_Power										As String 	= "ZPW"				' Zone2 Power 	
	Private Const DEFINE_Zone2_Volume										As String 	= "ZVL"				' Zone2 Volume 	
	Private Const DEFINE_Zone2_InputSelector								As String 	= "SLZ"				' Zone2 Input Selector

	Private Const DEFINE_Listening_Mode										As String 	= "LMD"				' Listening Mode
	
	Private 	  DEFINE_Responses_Values									As Map 
	

End Sub

Public  Sub Initialize(CallBack As Object, EventPrefix As String)
		   	mCallBack			= CallBack
    		mCallEventPrefix    = EventPrefix
			
			mConnected			= False
			
			DEFINE_Responses_Values.Initialize
			DEFINE_Responses_Values.Put(DEFINE_Master_Power,			"Master Power")
			DEFINE_Responses_Values.Put(DEFINE_Master_Volume,			"Master Volume")			
			DEFINE_Responses_Values.Put(DEFINE_Master_InputSelector,	"Master Input Selector")

			DEFINE_Responses_Values.Put(DEFINE_Zone2_Power,				"Zone 2 Power")
			DEFINE_Responses_Values.Put(DEFINE_Zone2_Volume,			"Zone 2 Volume")			
			DEFINE_Responses_Values.Put(DEFINE_Zone2_InputSelector,		"Zone 2 Input Selector")

			DEFINE_Responses_Values.Put(DEFINE_Listening_Mode,			"Listening Mode")			
End Sub

Private Sub MakeCmd(Cmd As String) As String
			Return mHeader &mDeviceID &Cmd &mTrailer			
End Sub

Private Sub MakePowerCmd(Master As Boolean, On As Boolean) As String
			If  Master Then
				If  On Then
					Return MakeCmd(DEFINE_Master_Power 	&DEFINE_Power_On)
				Else
					Return MakeCmd(DEFINE_Master_Power 	&DEFINE_Power_StandBy)
				End If
			Else
				If  On Then
					Return MakeCmd(DEFINE_Zone2_Power 	&DEFINE_Power_On)
				Else
					Return MakeCmd(DEFINE_Zone2_Power 	&DEFINE_Power_StandBy)
				End If				
			End If
End Sub

Private Sub MakeRaiseLowerVolumeCmd(Master As Boolean, Up As Boolean) As String
			If  Master Then
				If  Up Then
					Return MakeCmd(DEFINE_Master_Volume &DEFINE_Volume_Up)
				Else
					Return MakeCmd(DEFINE_Master_Volume &DEFINE_Volume_Down)
				End If
			Else
				If  Up Then
					Return MakeCmd(DEFINE_Zone2_Volume 	&DEFINE_Volume_Up)
				Else
					Return MakeCmd(DEFINE_Zone2_Volume 	&DEFINE_Volume_Down)
				End If				
			End If
End Sub

Private Sub MakeVolumeCmd(Volume As Int) As String
			Return MakeCmd(DEFINE_Master_Volume &Bit.ToHexString(Volume).ToUpperCase)
End Sub

Private Sub MakeSelectInputCmd(Master As Boolean, Input As String) As String

			If  cOnkyoCmds.FindInputSelectorByName(Input) Then
				If  Master Then
					Return MakeCmd(DEFINE_Master_InputSelector &cOnkyoCmds.GetInputSelectorByName(Input))
				Else
					Return MakeCmd(DEFINE_Zone2_InputSelector &cOnkyoCmds.GetInputSelectorByName(Input))
				End If
			Else
				If  Master Then
					Return MakeStatusCmd(DEFINE_Master_InputSelector)			
				Else				
					Return MakeStatusCmd(DEFINE_Zone2_InputSelector)			
				End If
			End If
End Sub

Private Sub MakeStatusCmd(MasterZone As String) As String 
			Return mHeader &mDeviceID &MasterZone &DEFINE_Status_Request &mTrailer			
End Sub

Private Sub MakeListeningModeCmd(ListeningMode As String) As String
'			Return mHeader &mDeviceID &DEFINE_Listening_Mode &Bit.ToHexString(ListeningMode).ToUpperCase &mTrailer		
			Return mHeader &mDeviceID &DEFINE_Listening_Mode &ListeningMode &mTrailer					
End Sub
Private Sub AlreadyConnected As ResumableSub  
			If  mConnected = False Or mSocket.Connected = False Then
				wait for (Connect) Complete(Connected As Boolean)
				
				Return True
			End If
			
			Return False
End Sub

Public  Sub Connect	As ResumableSub  
			mConnectTries = 0
			
			If  mSocket.IsInitialized = False Or mSocket.Connected = False Then
				mSocket.Initialize("cOnkyo_Socket")
				mSocket.Connect(mIPAddress, mPort, 0)	
				
				Do 	While mConnected = False
					Sleep(0)
				Loop
			Else
				mAStream.Close
				mSocket.Close
				
				mConnected = False
				
				If  mCallBack <> Null Then 
					If  SubExists(mCallBack, mCallEventPrefix &"_Connected") Then
						CallSubDelayed2(mCallBack, mCallEventPrefix &"_Connected", False)
					End If
				End If							
			End If			
			
			Return mConnected
End Sub

Public  Sub StatusCheck(Master As Boolean, Zone2 As Boolean)	
			If  Master Then
				wait for (SendCmd(Array As String(MakeStatusCmd(DEFINE_Master_Power), MakeStatusCmd(DEFINE_Master_Volume), MakeStatusCmd(DEFINE_Master_InputSelector), MakeStatusCmd(DEFINE_Listening_Mode)))) Complete
			End If
			
			If  Zone2 Then										 
				wait for (SendCmd(Array As String(MakeStatusCmd(DEFINE_Zone2_Power), MakeStatusCmd(DEFINE_Zone2_InputSelector)))) Complete
			End If
End Sub

Public  Sub SelectInput(Master As Boolean, Zone2 As Boolean, Input As String)
			If  Master Then 
				wait for (SendCmd(MakeSelectInputCmd(True, Input))) Complete				
			End If

			If  Zone2 Then 
				wait for (SendCmd(MakeSelectInputCmd(False, Input))) Complete				
			End If			
End Sub

Public  Sub VolumeUpDown(Up As Boolean)
			Wait for (SendCmd(MakeRaiseLowerVolumeCmd(True, Up))) Complete
End Sub

Public  Sub VolumeCommand(Input As String)
			wait for (SendCmd(MakeSelectInputCmd(True, Input))) Complete				
End Sub

Public  Sub PowerOn(Master As Boolean, Zone2 As Boolean)
			If  Master Then
				Wait for (SendCmd(MakePowerCmd(True, True))) Complete
			End If

			If  Master Then
				Wait for (SendCmd(MakePowerCmd(False, True))) Complete
			End If
End Sub				

Public  Sub PowerOff(Master As Boolean, Zone2 As Boolean)
			If  Master Then
				Wait for (SendCmd(MakePowerCmd(True, False))) Complete
			End If

			If  Master Then
				Wait for (SendCmd(MakePowerCmd(False, False))) Complete
			End If
End Sub				

Public  Sub TurnOnSetup(VolumeLevel As Int)
			Dim  Cmds	As List 
			
			Cmds.Initialize
			Cmds.AddAll(Array As String(MakePowerCmd(True, True), 				_ 
										MakePowerCmd(False, True), 				_
										MakeSelectInputCmd(True, "CD"), 		_
										MakeSelectInputCmd(False, "CD"),		_
										MakeStatusCmd(DEFINE_Master_Volume),	_
										MakeVolumeCmd(VolumeLevel),				_
										MakeListeningModeCmd("0C"),				_
										MakeStatusCmd(DEFINE_Listening_Mode)))	'  Set Listening Mode To ALL House Stereo
'			
			wait for (SendCmd(Cmds)) Complete		
End Sub

Public  Sub TurnOffSetup
			Dim  Cmds() As String = Array As String(MakeSelectInputCmd(True, "CBL/Sat"), 	_
													MakeListeningModeCmd("02"),				_
													MakeStatusCmd(DEFINE_Listening_Mode),	_ 
													MakePowerCmd(True, False), 				_
													MakePowerCmd(False, False)) 
			
			wait for (SendCmd(Cmds)) Complete
End Sub

Private Sub cOnkyo_Socket_Connected(Successful As Boolean)
			Log("Socket Connected:" &Successful)

			mConnected = Successful
									
			If  Successful = False Then
				mConnectTries = mConnectTries + 1
				
				If  mConnectTries > 10 Then			
					If  mCallBack <> Null Then 
						If  SubExists(mCallBack, mCallEventPrefix &"_Connected") Then
							CallSubDelayed2(mCallBack, mCallEventPrefix &"_Connected", False)
						End If
					End If							
					
					Return
				End If
				
				If  mSocket.IsInitialized Then 
					mSocket.Close
				End If
				
				mSocket.Initialize("cOnkyo_Socket")				
				mSocket.Connect(mIPAddress, mPort, 0)	
				Return
			End If
			
			ToastMessageShow("Connected to Onkyo", False)
			
			mAStream.Initialize(mSocket.InputStream, mSocket.OutputStream, "cOnkyo_AStream")
			
			If  mCallBack <> Null Then 
				If  SubExists(mCallBack, mCallEventPrefix &"_Connected") Then
					CallSubDelayed2(mCallBack, mCallEventPrefix &"_Connected", Successful)
				End If
			End If			
End Sub

Private Sub SendCmd(PassedValue As Object) As ResumableSub  
	
			Dim	Cmds 	As List
			
			Dim Cmd	 	As String
			Dim CmdPlus As String
			
			Dim Offset	As Int

			Cmds.Initialize
			
			If  PassedValue Is String Then
				Dim PV As String = PassedValue
				
				Cmds.Add(PV)
			Else
				If  GetType(PassedValue).StartsWith("[") Then
					Dim PAV() As String = PassedValue	
				
					Cmds.AddAll(PAV)
				Else
					Dim PL   As List	= PassedValue
					
					Cmds = PL
				End If
			End If
			
			
			Wait for (AlreadyConnected) Complete(ConnectedOnFly As Boolean)
						
			For i = 0 To Cmds.Size-1				
				Cmd 	= Cmds.Get(i)													'  Complete Command
				
				Offset 	= Cmd.IndexOf(mDeviceID)										'  Find where Device ID is
				
				If  Offset <> -1 Then
					CmdPlus = Cmd.SubString2(Offset, (Offset + (mDeviceID.Length + 5)))	'  Just the displayable portion
					Log("Sending (CmdPlus):" &CmdPlus)
				Else
					Log("Sending (Cmd)    :" &Cmd)
				End If
																				
				mAStream.Write(Cmd.GetBytes("utf8"))									'  Send the Command
				
				Sleep(250)																'  Give it sometime to respone
			Next
	
	
			If  ConnectedOnFly Then
				Log("Connected On Fly - Closing Connection")
				
				mAStream.Close
				mSocket.Close
				
				mConnected = False
			End If
			
			Return True		
End Sub

Private Sub cOnkyo_AStream_NewData(Buffer() As Byte)
			Dim FoundCmd 		As Boolean 	= False	
			
			Dim Offset			As Int 		
			
			Dim NewData			As String
			
			Dim DeviceID		As String
			
			Dim Response 		As String 	= BytesToString(Buffer, 0, Buffer.Length, "utf8")									
			Dim ResponseCmd 	As String
			Dim ResponseData	As String
			
			'-------------------------------------------------------------------------
			'  Process the Incoming Response (first trim off unneeded chars
			'-------------------------------------------------------------------------
			NewData 	= Response.Trim

			'-------------------------------------------------------------------------
			'  If Response Data does not start wit the ISCP header then what is it
			'-------------------------------------------------------------------------						
			If  NewData.StartsWith(mISCP) = False Then
				SendStatusMsg("Response Data[" &NewData &"]")
				Return
			End If

			'-------------------------------------------------------------------------
			'  Try and find the out device id in the response
			'-------------------------------------------------------------------------						
			Offset	= NewData.IndexOf(mDeviceID)
			
			'-------------------------------------------------------------------------
			'  If our device id is not there the maybe this command isn't for us
			'-------------------------------------------------------------------------
			If  Offset = -1 Then
				SendStatusMsg("Onkyo response string[" &NewData &"] not meant for us???")
				Return
			End If
			
			'-------------------------------------------------------------------------
			'  Save the Device ID and move onto the Command area
			'-------------------------------------------------------------------------			
			DeviceID = NewData.SubString2(Offset, (Offset + mDeviceID.Length))
			NewData	 = NewData.SubString(Offset + mDeviceID.Length)
			
			'-------------------------------------------------------------------------
			'  If the Command length is less then 3 characters something is wrong
			'-------------------------------------------------------------------------						
			If  NewData.Length < 3 Then
				SendStatusMsg("Onkyo response command string[" &NewData &"] for device[" &DeviceID &"] doesn't appear to be right!")
				Return				
			End If
			

			'-------------------------------------------------------------------------
			'  Save the Response Command and move the data 
			'-------------------------------------------------------------------------									
			ResponseCmd 	= NewData.SubString2(0, 3)
			ResponseData	= NewData.SubString(3)
			
			If  ResponseData.Length > 2 Then
				ResponseData = ResponseData.SubString2(0, 2)
			End If								
			
			
			If  DEFINE_Responses_Values.ContainsKey(ResponseCmd) Then
				Select 	ResponseCmd
						Case 	DEFINE_Master_Power, DEFINE_Zone2_Power
								FoundCmd = True
								'-------------------------------------------------------
								'  Power returns 00, 01 or N/A
								'-------------------------------------------------------
								If  ResponseData.StartsWith("00") Then
									SendStatusMsg(DEFINE_Responses_Values.Get(ResponseCmd) &": Standby")
								Else if ResponseData.StartsWith("01") Then									
										SendStatusMsg(DEFINE_Responses_Values.Get(ResponseCmd) &": ON")
								Else
										SendStatusMsg(DEFINE_Responses_Values.Get(ResponseCmd) &" NOT Found:[" &ResponseData &"]")									
								End If
											
						Case 	DEFINE_Master_Volume, DEFINE_Zone2_Volume
								FoundCmd = True								
										
								If  ResponseData.Contains("N/A") = False Then
									Try
										SendStatusMsg(DEFINE_Responses_Values.Get(ResponseCmd) &": " &Bit.ParseInt(ResponseData, 16))
									Catch
										SendStatusMsg("Response[" &ResponseData &"]")
									End Try
								End If
									
						Case 	DEFINE_Master_InputSelector, DEFINE_Zone2_InputSelector
								FoundCmd = True								
								
								If  cOnkyoCmds.FindInputSelectorByValue(ResponseData) Then
									SendStatusMsg(DEFINE_Responses_Values.Get(ResponseCmd) &": " &cOnkyoCmds.GetInputSelectorByValue(ResponseData))
								Else
									SendStatusMsg(DEFINE_Responses_Values.Get(ResponseCmd) &" NOT Found:[" &ResponseData &"]")									
								End If										

						Case 	DEFINE_Listening_Mode
								FoundCmd = True								
								'-------------------------------------------------------
								'  Selected Input return usually a 2 digit code 
								'-------------------------------------------------------
								If  cOnkyoCmds.FindListeningModeByValue(ResponseData) Then
									If  mListeningMode.EqualsIgnoreCase(ResponseData) = False Then
										mListeningMode = ResponseData										

										SendStatusMsg(DEFINE_Responses_Values.Get(ResponseCmd) &": " &cOnkyoCmds.GetListeningModeByValue(ResponseData))
									End If
								Else
									SendStatusMsg(DEFINE_Responses_Values.Get(ResponseCmd) &" NOT Found:[" &ResponseData &"]")									
								End If										
								
								
				End Select
			End If			
			
			If  FoundCmd = False Then
				Dim OnkyoCmd As TOnkyoCmd = cOnkyoCmds.FindCmd(ResponseCmd)

				SendStatusMsg("Onkyo Cmd[" &OnkyoCmd.Cmd &"] Verbose[" &OnkyoCmd.Verbose &"]"  &CRLF	_
							  &"Onkyo response command string[" &ResponseCmd &"][" &ResponseData &"] For device[" &DeviceID &"] NOT Handling!")
					
				Return				
			End If
End Sub

Private Sub SendStatusMsg(Msg As String)
			Log(Msg)
			
			
			If  mCallBack <> Null Then 
'				Log("SubCheck:" &mCallEventPrefix &"_StatusMessage")
				
				If  SubExists(mCallBack, mCallEventPrefix &"_StatusMessage") Then
					CallSubDelayed2(mCallBack, mCallEventPrefix &"_StatusMessage", Msg)
				End If
			End If			
End Sub

Private Sub cOnkyo_AStream_Error
			If  LastException.IsInitialized = True Then	
				Log("cOnkyo_AStream_Error" &LastException.Message)
			End If
End Sub

Private Sub cOnkyo_AStream_Terminated
			If  LastException.IsInitialized = True Then	
				Log("cOnkyo_AStream_Terminated" &LastException.Message)
			End If
End Sub

